@@ -1,24 +1,28 @@
 use ansi_term::Colour::*;
-use crate::{MismatchResult, display_result};
-use pact_matching::models::RequestResponseInteraction;
-use pact_matching::Mismatch;
 
-pub fn display_request_response_result(
+use pact_matching::Mismatch;
+use pact_models::sync_interaction::RequestResponseInteraction;
+
+use crate::{generate_display_for_result, MismatchResult};
+
+pub fn process_request_response_result(
   interaction: &RequestResponseInteraction,
-  match_result: &Result<Option<String>, MismatchResult>) {
+  match_result: &Result<Option<String>, MismatchResult>,
+  output: &mut Vec<String>) {
   match match_result {
     Ok(_) => {
-      display_result(
+      generate_display_for_result(
         interaction.response.status,
         Green.paint("OK"),
         interaction.response.headers.clone().map(|h| h.iter().map(|(k, v)| {
           (k.clone(), v.join(", "), Green.paint("OK"))
-        }).collect()), Green.paint("OK")
+        }).collect()), Green.paint("OK"),
+        output
       );
     },
     Err(ref err) => match *err {
       MismatchResult::Error(ref err_des, _) => {
-        println!("      {}", Red.paint(format!("Request Failed - {}", err_des)));
+        output.push(format!("      {}", Red.paint(format!("Request Failed - {}", err_des))));
       },
       MismatchResult::Mismatches { ref mismatches, .. } => {
         let status_result = if mismatches.iter().any(|m| m.mismatch_type() == "StatusMismatch") {
@@ -48,7 +52,7 @@ pub fn display_request_response_result(
           Green.paint("OK")
         };
 
-        display_result(interaction.response.status, status_result, header_results, body_result);
+        generate_display_for_result(interaction.response.status, status_result, header_results, body_result, output);
       }
     }
   }
